var gulp = require('gulp');
var gutil = require('gulp-util');
var bower = require('bower');
var concat = require('gulp-concat');
var sass = require('gulp-sass');
var minifyCss = require('gulp-minify-css');
var rename = require('gulp-rename');
var sh = require('shelljs');
var replace = require('gulp-replace');

var paths = {
  sass: ['./scss/**/*.scss']
};

gulp.task('default', ['sass']);

gulp.task('sass', function(done) {
  gulp.src('./scss/ionic.app.scss')
    .pipe(sass({
      errLogToConsole: true
    }))
    .pipe(gulp.dest('./www/css/'))
    .pipe(minifyCss({
      keepSpecialComments: 0
    }))
    .pipe(rename({ extname: '.min.css' }))
    .pipe(gulp.dest('./www/css/'))
    .on('end', done);
});

gulp.task('watch', function() {
  gulp.watch(paths.sass, ['sass']);
});

gulp.task('install', ['git-check'], function() {
  return bower.commands.install()
    .on('log', function(data) {
      gutil.log('bower', gutil.colors.cyan(data.id), data.message);
    });
});

gulp.task('git-check', function(done) {
  if (!sh.which('git')) {
    console.log(
      '  ' + gutil.colors.red('Git is not installed.'),
      '\n  Git, the version control system, is required to download Ionic.',
      '\n  Download git here:', gutil.colors.cyan('http://git-scm.com/downloads') + '.',
      '\n  Once git is installed, run \'' + gutil.colors.cyan('gulp install') + '\' again.'
    );
    process.exit(1);
  }
  done();
});


gulp.task('dev-urls', function(){
  gulp.src(['./www/js/app.js'])
  .pipe(replace('$scope.settings.wishlist_url', "'./amazon.html'"))
  .pipe(replace('$scope.settings.evergreen_url', "'./nccardinal.html'"))
  .pipe(gulp.dest('./www/js/'));
});

gulp.task('build-urls', function(){
  gulp.src(['./www/js/app.js'])
  .pipe(replace("'./amazon.html'", '$scope.settings.wishlist_url'))
  .pipe(replace("'./nccardinal.html'", '$scope.settings.evergreen_url'))
  .pipe(gulp.dest('./www/js/'));
});

gulp.task('dev-csp', function() {
  gulp.src('./www/index.html')
  .pipe(replace("<meta http-equiv=\"Content-Security-Policy\" " +
  "content=\"default-src *; style-src 'self' 'unsafe-inline'; script-src 'self' 'unsafe-inline' 'unsafe-eval'\"/>",
  "<!--<meta http-equiv=\"Content-Security-Policy\" " +
  "content=\"default-src *; style-src 'self' 'unsafe-inline'; script-src 'self' 'unsafe-inline' 'unsafe-eval'\"/>-->"))
  .pipe(gulp.dest('./www/'));
});

gulp.task('build-csp', function() {
  gulp.src('./www/index.html')
  .pipe(replace("<!--<meta http-equiv=\"Content-Security-Policy\" " +
  "content=\"default-src *; style-src 'self' 'unsafe-inline'; script-src 'self' 'unsafe-inline' 'unsafe-eval'\"/>-->",
  "<meta http-equiv=\"Content-Security-Policy\" " +
  "content=\"default-src *; style-src 'self' 'unsafe-inline'; script-src 'self' 'unsafe-inline' 'unsafe-eval'\"/>"))
  .pipe(gulp.dest('./www/'));
});

gulp.task('build', ['build-urls', 'build-csp']);
gulp.task('dev', ['dev-urls', 'dev-csp']);